@@ -7,6 +7,7 @@
 #include <string.h>
 #include <stdlib.h>
 #include "fitsio2.h"
+#include "group.h"  /* needed for fits_get_cwd in file_create */
 
 #if defined(unix) || defined(__unix__)  || defined(__unix)
 #include <pwd.h>         /* needed in file_openfile */
@@ -317,73 +318,97 @@ int file_create(char *filename, int *handle)
     FILE *diskfile;
     int ii;
     char mode[4];
-
-#if defined(BUILD_HERA) 
-
-    /* special code to verify that the path to the file to be created */
-    /* is within the users data directory on Hera */
  
-    int status = 0, rootlen, slen;
-    char *cpos;
+    int status = 0, rootlen, rootlen2, slen;
+    char *cptr, *cpos;
     char cwd[FLEN_FILENAME], absURL[FLEN_FILENAME];
-    /* note that "/heradata/users/" is actually "/.hera_mountpnt/hera_users/"  */
-    char rootstring[]="/.hera_mountpnt/hera_users/";
-    char username[FLEN_FILENAME], userroot[FLEN_FILENAME];
+    char rootstring[256], rootstring2[256];
+    char username[FLEN_FILENAME], userroot[FLEN_FILENAME], userroot2[FLEN_FILENAME];
 
-    /* Get the current working directory */
-    fits_get_cwd(cwd, &status);  
-    slen = strlen(cwd);
-    if (cwd[slen-1] != '/') strcat(cwd,"/"); /* make sure the CWD ends with slash */
+    cptr = getenv("HERA_DATA_DIRECTORY");
+    if (cptr) {
+	/* This environment variable is defined in the Hera data analysis environment. */
+	/* It specifies the root directory path to the users data directories.  */
+	/* CFITSIO will verify that the path to the file that is to be created */
+	/* is within this root directory + the user's home directory name. */
 
-/*    printf("CWD = %s\n", cwd);  */
-
-    /* check that CWD string matches the rootstring */
-    rootlen = strlen(rootstring);
-    if (strncmp(rootstring, cwd, rootlen)) {
-       ffpmsg("invalid CWD: does not match Hera data directory");
-/*       ffpmsg(rootstring);  */
-       return(FILE_NOT_CREATED); 
-    } else {
-
-       /* get the user name from CWD (it follows the root string) */
-       strcpy(username, cwd+rootlen);  
-       cpos=strchr(username, '/');
-       if (!cpos) {
-          ffpmsg("invalid CWD: not equal to Hera data directory + username");
-/*          ffpmsg(cwd); */
-          return(FILE_NOT_CREATED); 
-       } else {
-          *(cpos+1) = '\0';   /* truncate user name string */
-
-          /* construct full user root name */
-          strcpy(userroot, rootstring);
-          strcat(userroot, username);
-          rootlen = strlen(userroot);
-
-          /* convert the input filename to absolute path relative to the CWD */
-          fits_relurl2url(cwd,  filename,  absURL, &status);
 /*
-          printf("username = %s\n", username);
-          printf("userroot = %s\n", userroot);
-          printf("filename = %s\n", filename);
-          printf("ABS = %s\n", absURL);
-*/
-          /* check that CWD string matches the rootstring */
+printf("env = %s\n",cptr);
+*/	
+        if (strlen(cptr) > 200)  /* guard against possible string overflows */
+	    return(FILE_NOT_CREATED); 
 
-          if (strncmp(userroot, absURL, rootlen)) {
-             ffpmsg("invalid filename: path not within user directory");
+	/* environment variable has the form "path/one/;/path/two/" where the */
+	/* second path is optional */
+
+	strcpy(rootstring, cptr);
+	cpos = strchr(rootstring, ';');
+	if (cpos) {
+	    *cpos = '\0';
+	    cpos++;
+	    strcpy(rootstring2, cpos);
+	} else {
+	  *rootstring2 = '\0';
+	}
 /*
-             ffpmsg(absURL);
-             ffpmsg(userroot);
+printf("%s, %s\n", rootstring, rootstring2);
+printf("CWD = %s\n", cwd); 
+printf("rootstring=%s, cwd=%s.\n", rootstring, cwd);
 */
-             return(FILE_NOT_CREATED); 
-          }
-       }
+	/* Get the current working directory */
+	fits_get_cwd(cwd, &status);  
+	slen = strlen(cwd);
+	if (cwd[slen-1] != '/') strcat(cwd,"/"); /* make sure the CWD ends with slash */
+
+
+	/* check that CWD string matches the rootstring */
+	rootlen = strlen(rootstring);
+	if (strncmp(rootstring, cwd, rootlen)) {
+	    ffpmsg("invalid CWD: does not match root data directory");
+	    return(FILE_NOT_CREATED); 
+	} else {
+
+	    /* get the user name from CWD (it follows the root string) */
+	    strncpy(username, cwd+rootlen, 50);  /* limit length of user name */
+	    cpos=strchr(username, '/');
+	    if (!cpos) {
+               ffpmsg("invalid CWD: not equal to root data directory + username");
+               return(FILE_NOT_CREATED); 
+	    } else {
+	        *(cpos+1) = '\0';   /* truncate user name string */
+
+		/* construct full user root name */
+		strcpy(userroot, rootstring);
+		strcat(userroot, username);
+		rootlen = strlen(userroot);
+
+		/* construct alternate full user root name */
+		strcpy(userroot2, rootstring2);
+		strcat(userroot2, username);
+		rootlen2 = strlen(userroot2);
+
+		/* convert the input filename to absolute path relative to the CWD */
+		fits_relurl2url(cwd,  filename,  absURL, &status);
+
+/*
+printf("username = %s\n", username);
+printf("userroot = %s\n", userroot);
+printf("userroot2 = %s\n", userroot2);
+printf("filename = %s\n", filename);
+printf("ABS = %s\n", absURL);
+*/
+		/* check that CWD string matches the rootstring or alternate root string */
+
+		if ( strncmp(userroot,  absURL, rootlen)  &&
+		   strncmp(userroot2, absURL, rootlen2) ) {
+		   ffpmsg("invalid filename: path not within user directory");
+		   return(FILE_NOT_CREATED); 
+		}
+	    }
+	}
+	/* if we got here, then the input filename appears to be valid */
     }
-    /* if we got here, then the input filename appears to be valid */
-
-#endif
-
+    
     *handle = -1;
     for (ii = 0; ii < NMAXFILES; ii++)  /* find empty slot in table */
     {
